/*
    Project: TCGINA
    Version: V1.27
    File   : tcgdebug.c

    Copyright (c) 2005-2009 Author of TCGINA DLL. All rights reserved.
    Public Key Fingerprint: 294B A769 4A0A CC05 DAE6 00DD FF47 8C72 4097 67CE

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include "tcgpriv.h"
#include "secmem.h"

#ifdef TC_DEBUG

static void TCG_DumpProfileInfo(PTCG_ENCRYPTED_PROFILE_INFO pepi)
{
    TC_DbgPrintf(TEXT("  iStatus          : %d\n"), (int)pepi->iStatus);
    TC_DbgPrintf(TEXT("  szUserName       : %s\n"), pepi->szUserName);
    TC_DbgPrintf(TEXT("  szDomain         : %s\n"), pepi->szDomain);
    TC_DbgPrintf(TEXT("  fWaitForNtUserDat: %d\n"), (int)pepi->fWaitForNtUserDat);
    TC_DbgPrintf(TEXT("  szNtUserDat      : %s\n"), pepi->szNtUserDat);
    TCG_DumpDriveLetters("  MountedDrives    : %s\n", pepi->dwMountedDrives);
#ifdef SUPPORT_NETWORK
    TC_DbgPrintf(TEXT("  szNDLocalName    : %s\n"), pepi->szNDLocalName);
    TC_DbgPrintf(TEXT("  szRemoteName     : %s\n"), pepi->szRemoteName);
#endif
}

#ifdef SUPPORT_SHARED_MEMORY

static void TCG_DumpSession(DWORD dwSessionId)
{
    LPTSTR pBuffer;
    DWORD  cbSize;

    TC_DbgPrintfA("WTS Session ID     : 0x%08x\n", (unsigned)dwSessionId);
    if (WTSQuerySessionInformation(WTS_CURRENT_SERVER_HANDLE,
      dwSessionId, WTSConnectState, &pBuffer, &cbSize)) {
        LPCSTR rgszState[] = {
          "WTSActive", "WTSConnected", "WTSConnectQuery",
          "WTSShadow", "WTSDisconnected", "WTSIdle",
          "WTSListen", "WTSReset", "WTSDown", "WTSInit"
        };
        INT iState = *(PINT)pBuffer;
        if (iState < 10) {
            TC_DbgPrintfA("  WTS Connect State: %s (%d)\n",
              rgszState[iState], iState);
        } else {
            TC_DbgPrintfA("  WTS Connect State: %d\n", iState);
        }
        WTSFreeMemory(pBuffer);
    }
    if (WTSQuerySessionInformation(WTS_CURRENT_SERVER_HANDLE,
        dwSessionId, WTSClientName, &pBuffer, &cbSize)) {
        TC_DbgPrintfW(L"  WTS Client Name  : %s\n", pBuffer);
        WTSFreeMemory(pBuffer);
    }
    if (WTSQuerySessionInformation(WTS_CURRENT_SERVER_HANDLE,
        dwSessionId, WTSDomainName, &pBuffer, &cbSize)) {
        TC_DbgPrintfW(L"  WTS Domain Name  : %s\n", pBuffer);
        WTSFreeMemory(pBuffer);
    }
    if (WTSQuerySessionInformation(WTS_CURRENT_SERVER_HANDLE,
        dwSessionId, WTSUserName, &pBuffer, &cbSize)) {
        TC_DbgPrintfW(L"  WTS User Name    : %s\n", pBuffer);
        WTSFreeMemory(pBuffer);
    }
    if (WTSQuerySessionInformation(WTS_CURRENT_SERVER_HANDLE,
        dwSessionId, WTSWinStationName, &pBuffer, &cbSize)) {
        TC_DbgPrintfW(L"  WTS Win Station Name: %s\n", pBuffer);
        WTSFreeMemory(pBuffer);
    }
}

static void TCG_DumpSessions(void)
{
    DWORD  dwSessionId;
    PWTS_SESSION_INFO pSessions;
    DWORD  nSessions;

    if (ProcessIdToSessionId(GetCurrentProcessId(), &dwSessionId)) {
        TC_DbgPrintfA("Current Session ID : 0x%08x\n", (unsigned)dwSessionId);
    }
    if (WTSEnumerateSessions(WTS_CURRENT_SERVER_HANDLE, 0, 1,
      &pSessions, &nSessions)) {
        DWORD i;
        for (i = 0; i < nSessions; i++) {
            TCG_DumpSession(pSessions[i].SessionId);
        }
        WTSFreeMemory(pSessions);
    }
}
#endif

VOID WINAPI TCG_DumpContext(PTCG_CONTEXT pCtx)
{
    TC_DbgPrintfA("Context:\n");
    TC_DbgPrintfA("  dwOSVersion      : 0x%08x\n", (unsigned)pCtx->dwOSVersion);
    TCG_DumpDriveLetters("  PreMountedDrives : %s\n", pCtx->dwPreMountedDrives);
    TCG_DumpDriveLetters("  ProtectedDrives  : %s\n", pCtx->dwProtectedDrives);
#ifdef SUPPORT_SHARED_MEMORY
    TC_DbgPrintfA("  dwMSCount        : %d\n", (int)pCtx->dwMSCount);
    TC_DbgPrintfA("  dwMSOptions      : %d\n", (int)pCtx->dwMSOptions);
    if (pCtx->dwMSCount) {
        if (TCG_LockSharedMemory()) {
            PTCG_ENCRYPTED_PROFILE_INFO pepi;
            for (pepi = pCtx->pProfileBegin; pepi != pCtx->pProfileEnd; pepi++) {
                if (pepi->iStatus) {
                    TCG_DumpProfileInfo(pepi);
                }
            }
            TCG_UnlockSharedMemory();
        }
    } else {
        TCG_DumpProfileInfo(pCtx->pProfileBegin);
    }
    TCG_DumpSessions();
#else
    TCG_DumpProfileInfo(pCtx->pProfileBegin);
#endif
}

VOID WINAPI TCG_DumpHeap(VOID)
{
#ifdef SEC_DEBUG
    SEC_HEAP_INFORMATION shi;

    if (sec_heap_status(&shi)) {
        TC_DbgPrintfA("Secure heap usage:\n"
          "  Allocation count      = %u\n"
          "  Allocation size       = %u\n"
          "  Max. allocation count = %u\n"
          "  Max. allocation size  = %u\n",
          shi.cAllocations, shi.cbAllocationSize,
          shi.cMaxAllocations, shi.cbMaxAllocationSize);
    }
#endif
}

void WINAPI TCG_DumpDriveLetters(const char *paszText, DWORD dwDrives)
{
    DWORD i;
    CHAR  szDrive[26+1];
    CHAR  *psz;

    psz = szDrive;
    for (i = 0; i < 26; i++) {
        if (dwDrives & (1<<i)) {
            *psz++ = 'A' + (CHAR)i;
        }
    }
    *psz = '\0';
    TC_DbgPrintfA(paszText, szDrive);
}

#endif
